--  Copyright (2008-2013) Cdric Coussinet (cedric.coussinet@nomoseed.net)
--
--  This program is free software: you can redistribute it and/or modify
--  it under the terms of the GNU Affero General Public License as published
--  by the Free Software Foundation, either version 3 of the License, or
--  (at your option) any later version.
--
--  This program is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
--  GNU Affero General Public License for more details.

--  You should have received a copy of the GNU Affero General Public License
--  along with this program. If not, see <http://www.gnu.org/licenses/>

with Nomo.Interpreter.Gen_Clusters_Base.Perceptive_Loading;
pragma Elaborate_All (Nomo.Interpreter.Gen_Clusters_Base.Perceptive_Loading);

with Nomo.Interpreter.Gen_Clusters_Base.Initialization;
pragma Elaborate_All (Nomo.Interpreter.Gen_Clusters_Base.Initialization);

with Nomo.Interpreter.Types_Directory.Relations;

with Nomo.Interpreter.Perception_Rules.Initialization;

package body Nomo.Interpreter.Perceptions_Clusters.Loading is

   use Perception_Rules.Initialization;

   package Local_Loading is new Perceptions_Clusters_Base.Perceptive_Loading (Initialize, Is_Equal);

   function Has_Space (This : in Perceptions_Cluster) return Boolean is
   begin
      return Local_Loading.Has_Space (Cluster_Base (This));
   end Has_Space;

   function Get_Perception (This        : in Perceptions_Cluster;
                            Information : in Positive_Integer) return access Perception_Rule is
      use Local_Loading;
   begin
      return Get_Perception (Cluster_Base (This), Information);
   end Get_Perception;

   package Local_Initialization is new Perceptions_Clusters_Base.Initialization;

   procedure Initialize (This            : in out Perceptions_Cluster;
                         Perception_Type : in Perception_Type_Index) is
      use Types_Directory.Relations;
      use Local_Initialization;
   begin
      This.Perception_Type := Perception_Type;
      This.Input_Type := Get_Input_Type (Perception_Type);
      Initialize (Cluster_Base (This));
   end Initialize;

   procedure Load (This             : in out Perceptions_Cluster;
                   Rule_Storage_Ptr : not null access constant Storages.Full_Rule_Storage) is
      use Local_Loading;
   begin
      Load (Cluster_Base (This), Rule_Storage_Ptr, This.Input_Type);
   end Load;

end Nomo.Interpreter.Perceptions_Clusters.Loading;
